<?php
/**
 * Navigation controller, handles the administrative functions for this component.
 *
 * @package Navigation
 * @since 0.1
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\Controller;
use Core\Forms\Form;
use Core\NotFoundException;
use Core\View;


class NavigationController extends Controller {

	public function __construct() {
		$this->accessstring = 'g:admin';
	}


	public function edit() {
		$view = $this->getView();
		$m    = new NavigationModel($this->getRequest()->getParameter(0));

		if (!$m->checkExists()) {
			throw new NotFoundException();
		}

		$form = Form::BuildFromModel($m);
		$form->set('callsmethod', 'NavigationController::_SaveHandler');

		// I only want non-fuzzy pages to display.
		//$views = PageModel::GetPagesAsOptions("fuzzy = 0");
		$views = PageModel::GetPagesAsOptions();

		// Get the entries for this model as well.
		$entries = $m->getLink('NavigationEntry', 'weight ASC');

		$view->mastertemplate = 'admin';
		$view->title = 'Edit ' . $m->get('name');
		$view->assignVariable('model', $m);
		$view->assignVariable('form', $form);
		$view->assignVariable('pages', $views);
		$view->assignVariable('entries', $entries);
		$view->templatename = '/pages/navigation/edit.tpl';
		$view->addControl('New Navigation Menu', '/Navigation/Create', 'add');
		//$view->addControl('Delete Menu', '/Content/Delete/' . $m->get('id'), 'delete');
		$view->addControl('Navigation Listings', '/Navigation', 'directory');
	}

	public function create() {
		$view = $this->getView();
		$m    = new NavigationModel();

		$form = Form::BuildFromModel($m);
		$form->set('callsmethod', 'NavigationController::_SaveHandler');

		// I only want non-fuzzy pages to display.
		//$views = PageModel::GetPagesAsOptions("fuzzy = 0");
		$views = PageModel::GetPagesAsOptions();

		$view->mastertemplate = 'admin';
		$view->title        = 'New Navigation Menu';
		$view->templatename = '/pages/navigation/create.tpl';
		$view->assignVariable('model', $m);
		$view->assignVariable('form', $form);
		$view->assignVariable('pages', $views);
		$view->addControl('Navigation Listings', '/Navigation', 'directory');
	}

	public function delete() {
		$view = $this->getView();
		$req  = $this->getRequest();
		$mid  = $req->getParameter(0);
		$m    = new NavigationModel($mid);

		if (!$req->isPost()) {
			return View::STATUS_BADREQUEST;
		}

		if (!$m->checkExists()) {
			throw new NotFoundException();
		}

		$m->delete();

		\core\redirect('/navigation');
	}

	public static function _SaveHandler(Form $form) {

		// Save the model
		$m = $form->getModel();
		$isNew = $m->checkIsNew();
		$m->save();

		// Save the widget too
		$widget = $m->getLink('Widget');
		$widget->set('title', $m->get('name'));
		$widget->set('baseurl', '/navigation/view/' . $m->get('id'));
		$widget->set('editurl', '/navigation/edit/' . $m->get('id'));
		$widget->set('deleteurl', '/navigation/delete/' . $m->get('id'));
		$widget->save();

		// Save all the entries
		$counter = 0;

		$entries = $_POST['entries'] ?? [];
		$newMap = [];

		foreach ($entries as $id => $dat) {

			if($dat['status'] === 'deleted') {
				// Mark this entry as deleted, skipped for new entries.
				if(strpos($id, 'new-') === 0) {
					continue;
				}

				$entry = new NavigationEntryModel($id);
				if($entry->navigationid == $m->id) {
					// Security check to only manage entries on this widget.
					$entry->delete();
				}
			}
			elseif($dat['status'] === 'new') {
				// New entry!
				$entry = new NavigationEntryModel();

				$entry->setFromArray([
					'weight'       => ++$counter,
					'navigationid' => $m->id,
					'parentid'     => $newMap[ $dat['parent']] ?? $dat['parent'],
					'type'         => $dat['type'],
					'baseurl'      => $dat['url'],
					'title'        => $dat['title'],
					'target'       => $dat['target'],
				]);

				$entry->save();

				// Save this in the new map for children.
				$newMap[ $id ] = $entry->id;
			}
			else {
				// Standard existing entry.
				$entry = new NavigationEntryModel($id);

				$entry->setFromArray([
					'weight'       => ++$counter,
					'parentid'     => $newMap[ $dat['parent']] ?? $dat['parent'],
					'type'         => $dat['type'],
					'baseurl'      => $dat['url'],
					'title'        => $dat['title'],
					'target'       => $dat['target'],
				]);

				if($entry->navigationid == $m->id) {
					// Security check to only manage entries on this widget.
					$entry->save();
				}
			}
		}

		\Core\set_message(($isNew ? 'Created' : 'Updated') . ' navigation menu successfully!', 'success');
		return 'BACK';
	}
}
