<?php
/**
 * Navigation widget, handles displaying the navigation menus.
 *
 * @package Navigation
 * @since 2018.09
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\View;
use Core\Widget;
use Core\Model;
use Core\Router;

/**
 * Widget to display children and optionally sibling pages based off this current one.
 */
class NavigationPagesWidget extends Widget {
	
	public $displaySettings = [
		'display_links' => [
			'type' => 'select',
			'value' => 'children',
			'options' => [
				'children' => 'Children Pages (up to 2 levels deep)',
				'immediate-children' => 'Children Pages (immediate children only)',
				'siblings' => 'Sibling Pages (pages with the same parent)',
				'siblings-and-children' => 'Sibling Pages and Children Pages (up to 2 levels deep)'
			],
			'title' => 't:STRING_NAVIGATIONPAGES_WIDGET_DISPLAY_LINKS',
			'description' => 't:MESSAGE_NAVIGATIONPAGES_WIDGET_DISPLAY_LINKS',
		],
	];
	
	
	/**
	 * This is a widget to display children of the current page
	 *
	 * The page is dynamic based on the currently viewed page.
	 *
	 * @return int
	 */
	public function view() {
		$view    = $this->getView();
		$model   = Router::$DefaultPageModel;
		$entries = [];
		
		if (!$model) {
			// Pages that are not defined in the Page system, (such as dynamic pages), cannot have children.
			return '';
		}
		
		switch($this->getDisplaySetting('display_links')) {
			case 'siblings':
				$getSiblings = true;
				$getChildren = false;
				$getNesting  = false;
				break;
			case 'children':
				$getSiblings = false;
				$getChildren = true;
				$getNesting  = true;
				break;
			case 'immediate-children':
				$getSiblings = false;
				$getChildren = true;
				$getNesting  = false;
				break;
			default:
				$getSiblings = true;
				$getChildren = true;
				$getNesting  = true;
		}
		
		if( $model->parenturl && $getSiblings ) {
			// Give me all the siblings of that baseurl.
			$siblings = PageModel::Find(
				['parenturl' => $model->parenturl, 'selectable' => 1],
				null, 
				'title'
			);
		}
		elseif( $getChildren ) {
			$siblings = PageModel::Find(
				['parenturl' => $model->baseurl, 'selectable' => 1],
				null, 
				'title'
			);
		}
		else{
			$siblings = [$model];
		}
		
		
		foreach($siblings as $page) {
			if(!$page->isPublished()) {
				// If any page is not published, do not display it in the navigation menu, even for administrators.
				continue;
			}

			$subEntries = [];
			
			if(
				$getNesting &&
				($page->baseurl === $model->baseurl || $page->parenturl === $model->baseurl)
			) {
				$subPages = PageModel::Find(
					['parenturl' => $page->baseurl, 'selectable' => 1],
					null,
					'title'
				);
				
				foreach($subPages as $subPage) {
					$classes = [];
					$classes[] = Core\str_to_url($subPage->baseurl) . '-link';
					$classes[] = Core\str_to_url($subPage->rewriteurl) . '-link';
					
					$subEntries[] = [
						'obj'      => $subPage,
						'children' => [],
						'class'    => implode(' ', $classes),
					];
				}
			}
			
			$classes = [];
			$classes[] = Core\str_to_url($page->baseurl) . '-link';
			$classes[] = Core\str_to_url($page->rewriteurl) . '-link';
			
			if($page->baseurl === $model->baseurl) {
				$classes[] = 'active';
			}
			
			
			$entries[] = [
				'obj'      => $page,
				'children' => $subEntries,
				'class'    => implode(' ', $classes),
			];
		}

		$view->assign('entries', $entries);
	}

	/**
	 * Get the path for the preview image for this widget.
	 *
	 * Should be an image of size 210x70, 210x140, or 210x210.
	 *
	 * @return string
	 */
	public function getPreviewImage(){
		return 'assets/images/previews/templates/widgets/navigation/navigation-siblings.png';
	}
}
